/*
 *  SPDX-FileCopyrightText: 2015 Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */

#include "logging_p.h"

Q_LOGGING_CATEGORY(ROCS_GENERAL, "org.kde.rocs.general", QtWarningMsg)
